@@ -1,4 +1,4 @@
-# Copyright (c) 2013, Red Hat, Inc. All rights reserved.
+# Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -31,6 +31,9 @@
 LMI service provider client library.
 """
 
+import pywbem
+import re
+
 from lmi.shell import LMIInstance
 from lmi.shell import LMIInstanceName
 from lmi.scripts.common.errors import LmiFailed
@@ -39,7 +42,12 @@ from lmi.scripts.common import get_logger
 
 LOG = get_logger(__name__)
 
-SERVICE_KINDS = {'all', 'enabled', 'disabled', 'oneshot'}
+SERVICE_KINDS = {'all', 'enabled', 'disabled'}
+
+REQUESTED_STATE_ENABLED = 2
+REQUESTED_STATE_DISABLED = 3
+
+RE_SUFFIX = re.compile(r'\.service$')
 
 def invoke_on_service(ns, method, service, description):
     """
@@ -55,11 +63,13 @@ def invoke_on_service(ns, method, service, description):
     """
     inst = get_service(ns, service)
     service = inst.Name
-    res = getattr(inst, method)()
-    if res == 0:
-        LOG().debug('%s service "%s" on hostname "%s"',
-                description, service, ns.hostname)
-    return res
+    (rval, _, errorstr) = getattr(inst, method)()
+    if rval == 0:
+        LOG().info('%s service "%s".', description, service)
+    elif errorstr:
+        LOG().error('Operation failed on service "%s": %s.',
+                service, errorstr)
+    return rval
 
 def list_services(ns, kind='enabled'):
     """
@@ -69,7 +79,6 @@ def list_services(ns, kind='enabled'):
 
         * 'enabled'  - list only enabled services
         * 'disabled' - list only disabled services
-        * 'oneshot'  - list only oneshot services
         * 'all'      - list all services
 
     :returns: Instances of ``LMI_Service``.
@@ -79,18 +88,20 @@ def list_services(ns, kind='enabled'):
         raise TypeError("kind must be a string")
     if not kind in SERVICE_KINDS:
         raise ValueError("kind must be one of %s" % SERVICE_KINDS)
-    for service in sorted(ns.LMI_Service.instances(), key=lambda i: i.Name):
-        if kind == 'disabled' and service.EnabledDefault != \
-                ns.LMI_Service.EnabledDefaultValues.Disabled:
-            continue
-        if kind == 'oneshot' and service.EnabledDefault != \
-                ns.LMI_Service.EnabledDefaultValues.NotApplicable:
-            continue
-        if kind == 'enabled' and service.EnabledDefault != \
-                ns.LMI_Service.EnabledDefaultValues.Enabled:
-            # list only enabled
-            continue
-        yield service
+    try:
+        for service in sorted(ns.LMI_Service.instances(client_filtering=True),
+                key=lambda i: i.Name):
+            if kind == 'disabled' and service.EnabledDefault != \
+                    ns.LMI_Service.EnabledDefaultValues.Disabled:
+                continue
+            if kind == 'enabled' and service.EnabledDefault != \
+                    ns.LMI_Service.EnabledDefaultValues.Enabled:
+                # list only enabled
+                continue
+            yield service
+    except pywbem.CIMError as err:
+        raise LmiFailed('Failed to get service "%s": %s'
+                % (service, err.args[1]))
 
 def start_service(ns, service):
     """
@@ -99,7 +110,7 @@ def start_service(ns, service):
     :param service: Service name.
     :type service: string or :py:class:`lmi.shell.LMIInstanceName`
     """
-    return invoke_on_service(ns, 'StartService', service, 'started')
+    return invoke_on_service(ns, 'StartService', service, 'Started')
 
 def stop_service(ns, service):
     """
@@ -108,7 +119,7 @@ def stop_service(ns, service):
     :param string service: Service name or instance.
     :type service: string or :py:class:`lmi.shell.LMIInstanceName`
     """
-    return invoke_on_service(ns, 'StopService', service, 'stopped')
+    return invoke_on_service(ns, 'StopService', service, 'Stopped')
 
 def restart_service(ns, service, just_try=False):
     """
@@ -123,7 +134,7 @@ def restart_service(ns, service, just_try=False):
     method_name = 'RestartService'
     if just_try:
         method_name = 'Try' + method_name
-    return invoke_on_service(ns, method_name, service, 'restarted')
+    return invoke_on_service(ns, method_name, service, 'Restarted')
 
 def reload_service(ns, service, force=False, just_try=False):
     """
@@ -144,7 +155,24 @@ def reload_service(ns, service, force=False, just_try=False):
             method_name = 'ReloadOrRestart'
     else:
         method_name = 'Reload'
-    return invoke_on_service(ns, method_name, service, 'reloaded')
+    return invoke_on_service(ns, method_name, service, 'Reloaded')
+
+def enable_service(ns, service, enable=True):
+    """
+    Enable or disable service.
+
+    :param service: Service name or instance.
+    :type service: string or :py:class:`lmi.shell.LMIInstanceName`
+    :param boolean enable: Whether the service should be enabled or
+        disabled. Enabled service is started on system boot.
+    """
+    if enable:
+        method_name = 'TurnServiceOn'
+        description = 'Enabled'
+    else:
+        method_name = 'TurnServiceOff'
+        description = 'Disabled'
+    return invoke_on_service(ns, method_name, service, description)
 
 def get_service(ns, service):
     """
@@ -153,24 +181,87 @@ def get_service(ns, service):
 
     :param string service: Service name.
     """
-    if isinstance(service, basestring):
-        cs = get_computer_system(ns)
-        iname = ns.LMI_Service.new_instance_name({
-            "Name": service,
-            "CreationClassName" : "LMI_Service",
-            "SystemName" : cs.Name,
-            "SystemCreationClassName" : cs.CreationClassName
-        })
-        inst = iname.to_instance()
-        if inst is None:
-            raise LmiFailed('No such service "%s"' % service)
-    elif isinstance(service, (LMIInstance, LMIInstanceName)):
-        inst = service
-        service = inst.Name
-        if isinstance(inst, LMIInstanceName):
-            inst = inst.to_instance()
-    else:
-        raise TypeError("service must be either string or ``LMIInstanceName``")
+    try:
+        if isinstance(service, basestring):
+            if not service.endswith('.service'):
+                service += '.service'
+            cs = get_computer_system(ns)
+            iname = ns.LMI_Service.new_instance_name({
+                "Name": service,
+                "CreationClassName" : "LMI_Service",
+                "SystemName" : cs.Name,
+                "SystemCreationClassName" : cs.CreationClassName
+            })
+            inst = iname.to_instance()
 
-    return inst
+        elif isinstance(service, (LMIInstance, LMIInstanceName)):
+            inst = service
+            service = inst.Name
+            if isinstance(inst, LMIInstanceName):
+                inst = inst.to_instance()
 
+        return inst
+
+    except pywbem.CIMError as err:
+        if err.args[0] == pywbem.CIM_ERR_NOT_FOUND:
+            raise LmiFailed('No such service "%s".' % service)
+        else:
+            raise LmiFailed('Failed to get service "%s": %s'
+                    % (service, err.args[1]))
+
+    raise TypeError("service must be either string or ``LMIInstanceName``")
+
+def get_status_string(ns, service):
+    """
+    Return human friendly status description.
+
+    :param service: Either a service instance or its name.
+    :returns: Status description. One of
+        ``{ OK, Running, Stopped - OK, Stopped - Error }``.
+    :rtype: string
+    """
+    service = get_service(ns, service)
+
+    status = []
+    # first check for common statuses
+    if ns.LMI_Service.OperationalStatusValues.Completed in \
+            service.OperationalStatus:
+        status.append('Stopped')
+    if ns.LMI_Service.OperationalStatusValues.OK in service.OperationalStatus:
+        if not status:
+            status.append('Running')
+        else:
+            status.append('OK')
+    elif ns.LMI_Service.OperationalStatusValues.Error in \
+            service.OperationalStatus:
+        status.append('Error')
+
+    if not status:
+        # build the status from formal names
+        status = [  ns.LMI_Service.OperationalStatusValues.value_name(val)
+                 for val in service.OperationalStatus]
+
+        if not status:
+            status.append = 'Unknown'
+
+    return ' - '.join(status)
+
+def get_enabled_string(ns, service):
+    """
+    Return human friendly string for enabled state.
+
+    :param service: Either a service instance of its name.
+    :returns: Status description. One of:
+        ``{ Yes, No, Static }``.
+    ``Static`` represents a service that can not be enabled or disabled,
+    and are run only if something depends on them. It lacks ``[Install]``
+    section.
+    :rtype: string
+    """
+    service = get_service(ns, service)
+
+    if service.EnabledDefault == ns.LMI_Service.EnabledDefaultValues.Enabled:
+        return "Yes"
+    if service.EnabledDefault == ns.LMI_Service.EnabledDefaultValues.Disabled:
+        return "No"
+    return "Static"
