@@ -1,4 +1,4 @@
-# Copyright (c) 2013, Red Hat, Inc. All rights reserved.
+# Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -31,10 +31,12 @@
 System service management.
 
 Usage:
-    %(cmd)s list [--all | --disabled | --oneshot]
+    %(cmd)s list [(--enabled | --disabled)]
     %(cmd)s show <service>
     %(cmd)s start <service>
     %(cmd)s stop <service>
+    %(cmd)s enable <service>
+    %(cmd)s disable <service>
     %(cmd)s restart [--try] <service>
     %(cmd)s reload <service>
     %(cmd)s reload-or-restart [--try] <service>
@@ -52,28 +54,37 @@ Commands:
                 instead.
 
 Options:
-    --all       List all services available.
+    --enabled   List only enabled services.
     --disabled  List only disabled services.
-    --oneshot   List only oneshot services.
     --try       Whether to abandon the operation if the service is not running.
 """
+import functools
 
 from lmi.scripts import service as srv
 from lmi.scripts.common import command
 
 class Lister(command.LmiInstanceLister):
-    PROPERTIES = ('Name', "Started", 'Status')
 
-    def execute(self, ns, _all, _disabled, _oneshot):
-        kind = 'enabled'
-        if _all:
-            kind = 'all'
+    DYNAMIC_PROPERTIES = True
+
+    def execute(self, ns, _enabled, _disabled):
+        kind = 'all'
+        if _enabled:
+            kind = 'enabled'
         elif _disabled:
             kind = 'disabled'
-        elif _oneshot:
-            kind = 'oneshot'
-        for service_inst in srv.list_services(ns, kind):
-            yield service_inst
+
+        columns = [
+                ('Name', lambda i: srv.RE_SUFFIX.sub('', i.Name)),
+                ('Status', lambda i: srv.get_status_string(ns, i))]
+        if kind == 'all':
+            columns.append(('Enabled', lambda i: srv.get_enabled_string(ns, i)))
+
+        def generator():
+            for service_inst in srv.list_services(ns, kind):
+                yield service_inst
+
+        return columns, generator()
 
 class Start(command.LmiCheckResult):
     CALLABLE = srv.start_service
@@ -83,6 +94,20 @@ class Stop(command.LmiCheckResult):
     CALLABLE = srv.stop_service
     EXPECT = 0
 
+class Enable(command.LmiCheckResult):
+    CALLABLE = srv.enable_service
+    EXPECT = 0
+
+    def transform_options(self, options):
+        options['<enable>'] = True
+
+class Disable(command.LmiCheckResult):
+    CALLABLE = srv.enable_service
+    EXPECT = 0
+
+    def transform_options(self, options):
+        options['<enable>'] = False
+
 class Restart(command.LmiCheckResult):
     CALLABLE = srv.restart_service
     EXPECT = 0
@@ -107,13 +132,17 @@ class ReloadOrRestart(command.LmiCheckResult):
         return srv.reload_service(ns, service, force=True, just_try=_try)
 
 class Show(command.LmiShowInstance):
-    CALLABLE = srv.get_service
-    PROPERTIES = (
-            'Name',
-            'Caption',
-            ('Enabled', lambda i: i.EnabledDefault == 2),
-            ('Active', 'Started'),
-            'Status')
+
+    DYNAMIC_PROPERTIES = True
+
+    def execute(self, ns, service):
+        columns = (
+                ('Name', lambda i: srv.RE_SUFFIX.sub('', i.Name)),
+                'Caption',
+                ('Enabled', lambda i: srv.get_enabled_string(ns, i)),
+                ('Status', lambda i: srv.get_status_string(ns, i)))
+
+        return columns, srv.get_service(ns, service)
 
 Service = command.register_subcommands(
         'Service', __doc__,
@@ -121,6 +150,8 @@ Service = command.register_subcommands(
         , 'show'    : Show
         , 'start'   : Start
         , 'stop'    : Stop
+        , 'enable'  : Enable
+        , 'disable' : Disable
         , 'restart' : Restart
         , 'reload'  : Reload
         , 'reload-or-restart' : ReloadOrRestart
